@@ -1063,7 +1063,7 @@ BodyPane :: set_article (const Article& a)
 
   if (_message)
     g_object_unref (_message);
-  _message = _cache.get_message (_article.parts);
+  _message = _cache.get_message (_article.get_part_mids());
   refresh ();
 
   _data.mark_read (_article);
@@ -1413,9 +1413,12 @@ namespace
     }
   }
 
-  std::string get_utf8_body (GMimeMessage * source)
+  std::string get_utf8_body (GMimeMessage * source,
+                             const char   * fallback_charset)
   {
     ForeachPartData tmp;
+    if (fallback_charset)
+      tmp.fallback_charset = fallback_charset;
     if (source)
       g_mime_message_foreach_part (source, get_utf8_body_foreach_part, &tmp);
     return tmp.body;
@@ -1505,7 +1508,7 @@ BodyPane :: create_followup_or_reply (bool is_reply)
     else
     {
       // get the entire body...
-      h = get_utf8_body (_message);
+      h = get_utf8_body (_message, group_charset);
 
       // remove the sig...
       int sig_index (0);
