@@ -647,7 +647,7 @@ void GUI :: do_add_subthreads_to_selection ()
 }
 void GUI :: do_select_article_body ()
 {
-  std::cerr << "FIXME " << LINE_ID << std::endl;
+  _body_pane->select_all ();
 }
 void GUI :: do_show_preferences_dialog ()
 {
@@ -1023,6 +1023,11 @@ void GUI :: do_delete_article ()
 {
   const std::set<const Article*> articles (_header_pane->get_nested_selection());
   _data.delete_articles (articles);
+
+  const Quark mid (_body_pane->get_message_id());
+  foreach_const (std::set<const Article*>, articles, it)
+    if ((*it)->message_id == mid)
+      _body_pane->clear ();
 }
 void GUI :: do_mark_article_read ()
 {
@@ -1326,6 +1331,11 @@ void GUI :: do_read_selected_group ()
   _header_pane->set_focus ();
   watch_cursor_off ();
 
+  // periodically save our state
+  _data.save_state ();
+  _prefs.save ();
+  _group_prefs.save ();
+
   // maybe update the tab
   if (is_action_active ("tabbed-layout"))
     activate_action ("jump-to-header-tab");
