@@ -1578,6 +1578,9 @@ HeaderPane :: HeaderPane (ActionManager       & action_manager,
 #if GTK_CHECK_VERSION(2,8,0)
   gtk_tree_view_set_fixed_height_mode (GTK_TREE_VIEW(w), true);
 #endif
+#if GTK_CHECK_VERSION(2,10,0)
+  gtk_tree_view_set_rubber_banding (GTK_TREE_VIEW(w), true);
+#endif
 
   g_signal_connect (w, "button-release-event", G_CALLBACK(on_button_pressed), this);
   g_signal_connect (w, "button-press-event", G_CALLBACK(on_button_pressed), this);
