@@ -147,6 +147,15 @@ SaveDialog :: response_cb (GtkDialog * dialog,
   gtk_widget_destroy (GTK_WIDGET(dialog));
 }
 
+namespace
+{
+  void entry_changed_cb (GtkEditable * ignored, gpointer radio_or_null)
+  {
+    if (radio_or_null)
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(radio_or_null), true);
+  }
+}
+
 SaveDialog :: SaveDialog (Prefs                       & prefs,
                           const GroupPrefs            & group_prefs,
                           const ServerRank            & server_rank,
@@ -217,6 +226,7 @@ SaveDialog :: SaveDialog (Prefs                       & prefs,
     if (path.empty())
       path = g_get_home_dir ();
     file_entry_set (w, path.c_str());
+    g_signal_connect (file_entry_gtk_entry(w), "changed", G_CALLBACK(entry_changed_cb), _save_custom_path_radio);
     gtk_widget_set_usize (GTK_WIDGET(w), 400, 0);
     w = gtk_button_new_with_mnemonic (_("_Help"));
     gtk_box_pack_start (GTK_BOX(h), w, false, false, 0);
