@@ -200,16 +200,6 @@ NNTP :: on_socket_response (Socket * sock, const StringView& line_in)
          state = CMD_NEXT;
          break;
 
-      case NO_POSTING:
-      case POSTING_FAILED:
-         state = CMD_FAIL;
-         break;
-
-      case GROUP_NONEXISTENT:
-         Log::add_err_va (_("Unable to set group: %s"), line.to_string().c_str());
-         state = CMD_FAIL;
-         break;
-
       case XOVER_FOLLOWS:
       case ARTICLE_FOLLOWS:
       case NEWGROUPS_FOLLOWS:
@@ -218,12 +208,25 @@ NNTP :: on_socket_response (Socket * sock, const StringView& line_in)
          _nntp_response_text = true;
          break;
 
+      case GROUP_NONEXISTENT:
+      case NO_POSTING:
+      case POSTING_FAILED:
       case AUTH_REJECTED:
       case NO_GROUP_SELECTED:
       case ERROR_CMD_NOT_UNDERSTOOD:
       case ERROR_CMD_NOT_SUPPORTED:
-      case NO_PERMISSION:
-         Log::add_err_va (_("Error: %s"), line.to_string().c_str());
+      case NO_PERMISSION: {
+         StringView tmp (_previous_command);
+         if (tmp.len>=2 && tmp.str[tmp.len-1]=='\n' && tmp.str[tmp.len-2]=='\r')
+           tmp.len -= 2;
+         Log::add_err_va (_("Error: Got \"%s\" from %s in response to \"%*.*s\""),
+           line.to_string().c_str(),
+           sock->get_address().c_str(),
+           tmp.len, tmp.len, tmp.str);
+         state = CMD_FAIL;
+         break;
+      }
+
       case NO_SUCH_ARTICLE_NUMBER:
       case NO_SUCH_ARTICLE:
          state = CMD_FAIL;
