@@ -21,6 +21,7 @@
 #include <iostream>
 #include <sstream>
 #include <string>
+#include <map>
 extern "C" {
   #include <glib.h>
 }
@@ -36,6 +37,7 @@ using namespace pan;
 namespace
 {
   typedef std::vector<Task*> tasks_t;
+  typedef std::map<int,Article::Part> number_to_part_t;
 
   struct MyContext
   {
@@ -43,13 +45,15 @@ namespace
     std::string text;
     std::string path;
     Article a;
-    Article::Part part;
+    number_to_part_t parts;
     tasks_t tasks;
     ArticleCache& cache;
     ArticleRead& read;
     const ServerRank& ranks;
     const GroupServer& gs;
     const StringView fallback_path;
+    size_t bytes;
+    size_t number;
 
     MyContext (ArticleCache& ac, ArticleRead& r,
                const ServerRank& rank, const GroupServer& g, const StringView& p):
@@ -59,11 +63,10 @@ namespace
       groups.clear ();
       text.clear ();
       path.clear ();
-      part.clear ();
-      a.parts.clear ();
-      a.author.clear ();
-      a.subject.clear ();
-      a.time_posted = 0ul;
+      parts.clear ();
+      a.clear ();
+      bytes = 0;
+      number = 0;
     }
   };
 
@@ -87,10 +90,11 @@ namespace
     }
 
     else if (!strcmp (element_name, "segment")) {
-      mc.part.clear ();
+      mc.bytes = 0;
+      mc.number = 0;
       for (const char **k(attribute_names), **v(attribute_vals); *k; ++k, ++v) {
-             if (!strcmp (*k,"bytes"))  mc.part.bytes = strtoul (*v,0,10);
-        else if (!strcmp (*k,"number")) mc.part.index = atoi (*v);
+             if (!strcmp (*k,"bytes"))  mc.bytes = strtoul (*v,0,10);
+        else if (!strcmp (*k,"number")) mc.number = atoi (*v);
       }
     }
   }
@@ -106,25 +110,31 @@ namespace
     if (!strcmp(element_name, "group"))
       mc.groups.insert (Quark (mc.text));
 
-    else if (!strcmp(element_name, "segment") && (mc.part.index>0) && !mc.text.empty()) {
-      mc.part.message_id = Quark ("<" + mc.text + ">");
-      if (mc.a.parts.size() < mc.part.index)
-        mc.a.set_part_count (mc.part.index);
-      mc.a.add_part (mc.part);
-      mc.part.clear ();
+    else if (!strcmp(element_name, "segment") && mc.number && !mc.text.empty()) {
+      const std::string mid ("<" + mc.text + ">");
+      if (mc.a.message_id.empty())
+          mc.a.message_id = mid;
+      Article::Part& part (mc.parts[mc.number]);
+      part.bytes = mc.bytes;
+      part.set_message_id (mc.a.message_id, mid);
     }
 
     else if (!strcmp(element_name,"path"))
       mc.path = mc.text;
 
-    else if (!mc.a.parts.empty() && !strcmp (element_name, "file")) {
+    else if (!mc.parts.empty() && !strcmp (element_name, "file"))
+    {
+      // populate mc.a.parts
+      mc.a.set_part_count (mc.parts.rbegin()->first);
+      foreach (number_to_part_t, mc.parts, it)
+        mc.a.get_part(it->first).swap (it->second);
+
       foreach_const (quarks_t, mc.groups, git) {
         quarks_t servers;
         mc.gs.group_get_servers (*git, servers);
         foreach_const (quarks_t, servers, sit)
           mc.a.xref.insert (*sit, *git, 0);
       }
-      mc.a.message_id = mc.a.parts[0].message_id;
       const StringView p (mc.path.empty() ? mc.fallback_path : StringView(mc.path));
       mc.tasks.push_back (new TaskArticle (mc.ranks, mc.gs, mc.a, mc.cache, mc.read, 0, TaskArticle::DECODE, p));
     }
@@ -249,21 +259,26 @@ NZB :: nzb_to_xml (std::ostream             & out,
 
     // now for the parts...
     out << indent(depth++) << "<segments>\n";
+    int part_number (0);
     foreach_const (Article::parts_t, a.parts, pit)
     {
+      ++part_number;
+
       // incomplete multipart...
       if (pit->empty())
         continue;
 
       // remove the surrounding < > as per nzb spec
-      StringView mid (pit->message_id);
-      if (!mid.empty() && mid.str[0]=='<')
-        mid = mid.substr (mid.begin()+1, mid.end()-1);
+      std::string mid (pit->get_message_id (a.message_id));
+      if (mid.size()>=2 && mid[0]=='<') {
+        mid.erase (0, 1);
+        mid.resize (mid.size()-1);
+      }
 
       // serialize this part
       out << indent(depth)
           << "<segment" << " bytes=\"" << pit->bytes << '"'
-                        << " number=\"" << pit->index << '"'
+                        << " number=\"" << part_number << '"'
                         << ">";
       escaped(out, mid);
       out  << "</segment>\n";
