@@ -57,12 +57,14 @@ namespace pan
       void reset_speed_counter ();
       void set_abort_flag (bool b);
       bool is_abort_set () const;
+      std::string get_address () const { return _address; }
 
     protected:
       void increment_xfer_byte_count (unsigned long byte_count);
       unsigned long _byte_count;
       time_t _time_started;
       bool _abort_flag;
+      std::string _address;
 
     public:
 
