@@ -76,7 +76,7 @@ TaskArticle :: TaskArticle (const ServerRank          & server_rank,
   _finished_proc_has_run (false),
   _save_mode (save_mode)
 {
-  cache.reserve (article.parts);
+  cache.reserve (article.get_part_mids());
 
   if (listener != 0)
     add_listener (listener);
@@ -98,7 +98,9 @@ TaskArticle :: TaskArticle (const ServerRank          & server_rank,
   foreach_const (Article::parts_t, article.parts, it)
   {
     all_bytes += it->bytes;
-    if (!it->empty() && !cache.contains (it->message_id))
+
+    const std::string mid (it->get_message_id (article.message_id));
+    if (!it->empty() && !cache.contains (mid))
     {
       need_bytes += it->bytes;
 
@@ -110,7 +112,7 @@ TaskArticle :: TaskArticle (const ServerRank          & server_rank,
       // `ARTICLE message-id' instead when talking to the server.
       foreach_const (quarks_t, servers, sit)
         foreach_const (quarks_t, groups, git)
-          n.xref.insert (*sit, *git, it->message_id==article.message_id.to_string() ? article.xref.find_number(*sit,*git) : 0);
+          n.xref.insert (*sit, *git, mid==article.message_id.to_string() ? article.xref.find_number(*sit,*git) : 0);
 
       _needed.push_back (n);
     }
@@ -129,7 +131,7 @@ TaskArticle :: TaskArticle (const ServerRank          & server_rank,
 
 TaskArticle :: ~TaskArticle ()
 {
-  _cache.release (_article.parts);
+  _cache.release (_article.get_part_mids());
 }
 
 void
@@ -202,7 +204,7 @@ TaskArticle :: use_nntp (NNTP * nntp)
     if (number)
       nntp->article (group, number, this);
     else
-      nntp->article (group, needed->part.message_id.c_str(), this);
+      nntp->article (group, needed->part.get_message_id(_article.message_id).c_str(), this);
     update_work ();
   }
 }
@@ -246,7 +248,7 @@ TaskArticle :: on_nntp_done  (NNTP    * nntp,
 
   if (health == OK) { // if download succeeded, save it in the cache
     const StringView view (&it->buf.front(), it->buf.size());
-    _cache.add (it->part.message_id, view);
+    _cache.add (it->part.get_message_id(_article.message_id), view);
     ++_stats[nntp->_server];
   }
 
@@ -282,9 +284,11 @@ namespace
 void
 TaskArticle :: on_finished ()
 {
+  const Article::mid_sequence_t mids (_article.get_part_mids());
+  const ArticleCache :: strings_t filenames (_cache.get_filenames (mids));
+
   if (_save_mode & RAW)
   {
-    const ArticleCache :: strings_t filenames (_cache.get_filenames (_article.parts));
     foreach_const (ArticleCache::strings_t, filenames, it)
     {
       gchar * contents (0);
@@ -310,8 +314,6 @@ TaskArticle :: on_finished ()
 
   if (_save_mode & DECODE)
   {
-    const ArticleCache :: strings_t filenames (_cache.get_filenames (_article.parts));
-
     // decode
     int res;
     if (((res = UUInitialize())) != UURET_OK)
